﻿//////////////////////////////////////////////
// ProgramSourcesHolder.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkMemory
#include <NilkinsMemory/Containers/BufferCast.h>
#include <NilkinsMemory/Containers/String.h>
#include <NilkinsMemory/Containers/StringView.h>

/// Internals --------------------------------

namespace nkGraphics
{
	struct DLL_GRAPHICS_EXPORT MacroEntry
	{
		nkMemory::String _key ;
		nkMemory::String _value ;
	} ;
}

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ProgramSourcesHolder final
	{
		private :

			// Les mémoires
			// Ce seront des char* pour éviter les différences 
			nkMemory::String _vertexMemory ;
			nkMemory::String _hullMemory ;
			nkMemory::String _domainMemory ;
			nkMemory::String _geometryMemory ;
			nkMemory::String _pixelMemory ;
			nkMemory::String _computeMemory ;
			nkMemory::String _raytracingMemory ;

			nkMemory::BufferCast<MacroEntry> _macrosVertex ;
			nkMemory::BufferCast<MacroEntry> _macrosHull ;
			nkMemory::BufferCast<MacroEntry> _macrosDomain ;
			nkMemory::BufferCast<MacroEntry> _macrosGeometry ;
			nkMemory::BufferCast<MacroEntry> _macrosPixel ;
			nkMemory::BufferCast<MacroEntry> _macrosCompute ;
			nkMemory::BufferCast<MacroEntry> _macrosRaytracing ;

		public :

			// Constructeur, destructeur
			ProgramSourcesHolder () ;
			~ProgramSourcesHolder () ;

			// Getters
			nkMemory::StringView getVertexMemory () const ;
			nkMemory::StringView getHullMemory () const ;
			nkMemory::StringView getDomainMemory () const ;
			nkMemory::StringView getGeometryMemory () const ;
			nkMemory::StringView getPixelMemory () const ;
			nkMemory::StringView getComputeMemory () const ;
			nkMemory::StringView getRaytracingMemory () const ;
			const nkMemory::BufferCast<MacroEntry>& getDefineMacrosVertex () const ;
			const nkMemory::BufferCast<MacroEntry>& getDefineMacrosHull () const ;
			const nkMemory::BufferCast<MacroEntry>& getDefineMacrosDomain () const ;
			const nkMemory::BufferCast<MacroEntry>& getDefineMacrosGeometry () const ;
			const nkMemory::BufferCast<MacroEntry>& getDefineMacrosPixel () const ;
			const nkMemory::BufferCast<MacroEntry>& getDefineMacrosCompute () const ;
			const nkMemory::BufferCast<MacroEntry>& getDefineMacrosRaytracing () const ;
			unsigned int getDefineMacroMaxNumber () const ;
			unsigned int getDefineMacroMaxNumberPipeline () const ;
			unsigned int getDefineMacroNumberCompute () const ;
			unsigned int getDefineMacroNumberRaytracing () const ;

			// Setters
			void setVertexMemory (const nkMemory::StringView& str) ;
			void setHullMemory (const nkMemory::StringView& str) ;
			void setDomainMemory (const nkMemory::StringView& str) ;
			void setGeometryMemory (const nkMemory::StringView& str) ;
			void setPixelMemory (const nkMemory::StringView& str) ;
			void setComputeMemory (const nkMemory::StringView& str) ;
			void setRaytracingMemory (const nkMemory::StringView& str) ;
			void addDefineMacroVertex (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroHull (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroDomain (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroGeometry (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroPixel (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroCompute (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroRaytracing (const nkMemory::StringView& key, const nkMemory::StringView& value) ;

			// Utilitaires
			bool hasMinimalDefinitionsPipeline () const ;
			bool hasMinimalDefinitionsCompute () const ;
			bool hasMinimalDefinitionsGeometry () const ;
			bool hasMinimalDefinitionsRaytracing () const ;
			void copyDefinesFrom (const ProgramSourcesHolder& other) ;
	} ;
}