﻿//////////////////////////////////////////////
// RenderQueueManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Entity ;
	class PostProcessPass ;
	class RenderQueue ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>
#include <vector>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT RenderQueueManager final : public nkCommon::SingletonClass<RenderQueueManager>
	{
		friend class nkCommon::SingletonClass<RenderQueueManager> ;

		private :
		
			// Attributs
			// Mutex
			mutable std::recursive_mutex _mutex ;
			// Memory
			std::unordered_map<std::string, RenderQueue*> _queues ;

			// Rendering rq
			RenderQueue* _renderingRQ ;

		private :

			// Functions
			// Constructeur
			RenderQueueManager () ;

		public :

			// Constants
			inline static const nkMemory::StringView DEFAULT_RENDER_QUEUE = "NILKINS_DEFAULT_RENDER_QUEUE" ;
		
		public :

			// Destructeur
			~RenderQueueManager () ;
			
			// Management
			RenderQueue* createOrRetrieve (const nkMemory::StringView& name) ;
			RenderQueue* get (const nkMemory::StringView& name) const ;
			RenderQueue* getByIndex (unsigned int index) const ;
			void rename (const nkMemory::StringView& oldName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;

			// Rendering
			void updateRenderingQueue (const nkMemory::StringView& name) ;
			void updateRenderingQueue (RenderQueue* fromThisOne) ;
			RenderQueue* getComputedRenderingQueue () const ;
	} ;
}